<?php

namespace Drupal\mcapi_signatures\Plugin\TransactionRelative;

use Drupal\mcapi\Plugin\TransactionRelativeInterface;
use Drupal\mcapi\Entity\TransactionInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Database\Query\Condition;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
/**
 * Defines a payee relative to a Transaction entity.
 *
 * @TransactionRelative(
 *   id = "pending_signatory",
 *   label = @Translation("Pending signatory, (may cause duplicate results in views)"),
 *   description = @Translation("Users whose signature the transaction is awaiting")
 * )
 */
class PendingSignatory extends PluginBase implements TransactionRelativeInterface, ContainerFactoryPluginInterface {

  private $database;

  /**
   * Constructor.
   */
  public function __construct($configuration, $plugin_id, $plugin_definition, $database) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->database = $database;
  }

  /**
   * Injector.
   */
  static public function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('database')
    );
  }

  public function isRelative(TransactionInterface $transaction, AccountInterface $account) : bool {
    if (isset($transaction->signatures[$account->id()])) {
      return empty($transaction->signatures[$account->id()]);
    }
    return FALSE;
  }


  /**
   * {@inheritdoc}
   */
  public function queryCondition(SelectInterface $query, Condition $or_group, $uid) {
    $query->join(
      'mcapi_wallet',
      'signature_wallets',
      'mcapi_transaction.payer = signature_wallets.wid OR mcapi_transaction.payee = signature_wallets.wid'
    );
    $query->join(
      'mcapi_wallet__holder',
      'wallet_holder',
      "wallet_holder.entity_id = signature_wallets.wid AND wallet_holder.holder_target_type = 'user'"
    );
    $query->join(
      'users',
      'holder_users',
      "holder_users.uid = wallet_holder.holder_target_id"
    );
    $query->join(
      'mcapi_signatures',
      'signatories',
      'signatories.uid = holder_users.uid AND signatories.signed = 0'
    );
    $or_group->condition('signatories.uid', $uid);
  }

  /**
   * {@inheritdoc}
   */
  public function queryConditionIndex(SelectInterface $query, Condition $or_group, $uid) {
    $query->join(
      'mcapi_wallet',
      'signature_wallets',
      'mcapi_transactions_index.wallet_id = signature_wallets.wid'
    );
    $query->join(
      'mcapi_wallet__holder',
      'wallet_holder',
      "wallet_holder.entity_id = signature_wallets.wid AND wallet_holder.holder_target_type = 'user'"
    );
    $query->join(
      'users',
      'holder_users',
      "holder_users.uid = wallet_holder.holder_target_id"
    );
    $query->join(
      'mcapi_signatures',
      'signatories',
      'signatories.uid = holder_users.uid AND signatories.signed = 0'
    );
    $or_group->condition('signatories.uid', $uid);
  }


  /**
   * {@inheritdoc}
   */
  public function getUsers(TransactionInterface $transaction) : array {
    // @todo inject the database connection
    return $this->database->select('mcapi_signatures', 's')->fields('s', ['uid'])
      ->condition('serial', $transaction->serial->value)
      ->condition('signed', 0)
      ->execute()->fetchCol();
  }


}
