<?php

namespace Drupal\mcapi\Entity\ListBuilder;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a class to build a listing of wallet entities.
 *
 */
class WalletListBuilder extends EntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    // Enable language column and filter if multiple languages are added.
    $header = [
      'name' => [
        'data' => $this->t('Name')
      ],
      'balance' => $this->t('Balance'),
      'trades' => [
        'data' => $this->t('Trades'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      'holder' => [
        'data' => $this->t('Held by'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      'operations' => $this->t('Operations'),
    ];
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['id']['data'] = [
      '#type' => 'link',
      '#title' => $entity->label() .'(#'.$entity->id().')',
      '#url' => $entity->toUrl(),
    ];

    $row['balance']['data'] = [
      '#markup' => $entity->balance,
    ];
    $row['trades']['data'] = [
      '#markup' => $entity->trades->getValue(),
    ];
    $row['holder'] = $entity->holder->entity? $entity->holder->entity->toLink() : 'orphan';
    $row['operations']['data'] = $this->buildOperations($entity);
    return $row + parent::buildRow($entity);
  }

}
