<?php

namespace Drupal\mcapi\Entity\Storage;

use Drupal\mcapi\Entity\Wallet;
use Drupal\user\Entity\User;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\Sql\SqlContentEntityStorage;
use Drupal\field\Entity\FieldStorageConfig;


/**
 * Storage controller for wallet entity.
 */
class WalletStorage extends SqlContentEntityStorage implements WalletStorageInterface {

  /**
   * {@inheritdoc}
   */
  public function getQueryServiceName() {
    return 'mcapi_wallet.query.sql';
  }

  /**
   * {@inheritdoc}
   */
  public static function walletsOf(ContentEntityInterface $entity, $load = FALSE) {
    // This entityQuerys have no accessCheck because wallet view acces basically
    // depends only on permissions and on the holder status, and we already
    // decided who the holder is.
    $wids = \Drupal::entityQuery('mcapi_wallet')
      ->accessCheck(FALSE)
      ->condition('holder.target_type', $entity->getEntityTypeId())
      ->condition('holder.target_id', $entity->id())
      ->execute();
    return $load ?
      Wallet::loadMultiple($wids) :
      $wids;
  }

  /**
   * Return the first wallet held by a given entity.
   *
   * @param ContentEntityBase $entity
   *   Defaults to the current user.
   * @return int
   *   0 if no wallets are held
   */
  static function firstWalletOfEntity(ContentEntityInterface $entity) : int {
    if ($wids = self::walletsOf($entity)) {
      return reset($wids);
    }
    return 0;
  }

  static function hasWallet(ContentEntityInterface $entity) : bool {
    return (bool)static::walletsOf($entity, FALSE);
  }

}

