<?php

namespace Drupal\mcapi\Event;

use Drupal\mcapi\Entity\Transaction;
use Symfony\Component\EventDispatcher\GenericEvent;

/**
 * Event which is fired before a new transaction is validated.
 *
 * @see \Drupal\mcapi\Entity\Transaction::assemble()
 */
class TransactionAssembleEvent extends GenericEvent {

  const EVENT_NAME = 'mcapi_transaction.assemble';

  /**
   * Add a child transaction.
   *
   * @param array $props
   *   The new child transaction.
   */
  public function addChildTransaction(array $props) {
    $transaction = $this->getTransaction();
    if ($props['payer'] == $props['payee']) {
      \Drupal::logger('Community Accounting')->alert(
        'Skipped auto transaction with same payer and payee: @name',
        ['@name' => $props['payee']]
      );
      return;
    }
    // References \Drupal\mcapi\Entity\Type.
    $props['type'] = 'auto';
    $props['state'] = $transaction->state->target_id;
    $transaction->children[] = Transaction::create($props);
  }

  /**
   * Get the parent transaction.
   *
   * @return Transaction
   */
  public function getTransaction() {
    return $this->getSubject();
  }


}
