<?php

namespace Drupal\mcapi\Plugin\TransactionRelative;

use Drupal\mcapi\Entity\TransactionInterface;
use Drupal\mcapi\Plugin\TransactionRelativeInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Database\Query\Condition;

/**
 * Defines a creator relative to a Transaction entity.
 *
 * @TransactionRelative(
 *   id = "creator",
 *   label = @Translation("Creator"),
 *   description = @Translation("The user who created the transaction")
 * )
 */
class Creator extends PluginBase implements TransactionRelativeInterface {

  /**
   * {@inheritdoc}
   */
  public function isRelative(TransactionInterface $transaction, AccountInterface $account) : bool {
    return $transaction->creator->entity->id() == $account->id();
  }

  /**
   * {@inheritdoc}
   */
  public function queryCondition(SelectInterface $query, Condition $or_group, $uid) {
    $transaction_table = $query->getTables()['mcapi_transaction'];
    $or_group->condition($transaction_table['alias'].'.creator', $uid);
  }

  public function queryConditionIndex(SelectInterface $query, Condition $or_group, $uid) {
    $transaction_table = $query->getTables()['mcapi_transaction'];
    $or_group->condition($transaction_table['alias'].'.creator', $uid);
  }

  /**
   * {@inheritdoc}
   */
  public function getUsers(TransactionInterface $transaction) : array {
    return [$transaction->getOwnerId()];
  }

}
