<?php

namespace Drupal\mcapi\Plugin\TransactionRelative;

use Drupal\mcapi\Plugin\TransactionRelativeInterface;
use Drupal\mcapi\Entity\TransactionInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Database\Query\Condition;
use Drupal\Core\Database\Query\SelectInterface;

/**
 * Defines a payer relative to a Transaction entity.
 *
 * @TransactionRelative(
 *   id = "payer",
 *   label = @Translation("Owner of payer wallet"),
 *   description = @Translation("The owner of the payer wallet")
 * )
 */
class Payer extends PluginBase implements TransactionRelativeInterface {

  /**
   * {@inheritdoc}
   */
  public function isRelative(TransactionInterface $transaction, AccountInterface $account) : bool {
    return $transaction->payer->entity->getOwner()->id() == $account->id();
  }

  /**
   * {@inheritdoc}
   */
  public function queryCondition(SelectInterface $query, Condition $or_group, $uid) {
    $transaction_table = $query->getTables()['mcapi_transaction'];
    $this->joinWalletTofield($query, $transaction_table['alias'].'.payer');
    $or_group->condition('payer_user.uid', $uid);
  }

  /**
   * {@inheritdoc}
   */
  public function queryConditionIndex(SelectInterface $query, Condition $or_group, $uid) {
    $transaction_table = $query->getTables['mcapi_transactions_index'];
    $this->joinWalletTofield($query, $transaction_table['alias'].'.uid1');
    $or_group->condition('payer_user.uid', $uid);
  }

  /**
   * {@inheritdoc}
   */
  private function joinWalletTofield(SelectInterface $query, $field_alias) {
    $query->join('mcapi_wallet', 'payer_wallet', "$field_alias = payer_wallet.wid");
    $query->join('mcapi_wallet__holder', 'payer_holder', 'payer_holder.entity_id = payer_wallet.wid');
    $query->join('users', 'payer_user', "payer_holder.holder_target_id = payer_user.uid");
  }

  /**
   * {@inheritdoc}
   */
  public function getUsers(TransactionInterface $transaction) : array {
    return [$transaction->payer->entity->getOwner()->id()];
  }

}
