<?php

namespace Drupal\mcapi\Plugin\views\argument_validator;

use Drupal\views\Plugin\views\argument_validator\ArgumentValidatorPluginBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Validates whether user access is allowed.
 *
 * @ViewsArgumentValidator(
 *   id = "wallet_private",
 *   title = @Translation("Wallet private"),
 *   entity_type = "mcapi_wallet"
 * )
 * @note this isn't used in any of the default views.
 */
class WalletPrivate extends ArgumentValidatorPluginBase {

  protected $walletStorage;

  /**
   * @param array $configuration
   * @param string $plugin_id
   * @param array $plugin_definition
   * @param EntityTypeManagerInterface $entity_type_manager
   * @param EntityTypeBundleInfoInterface $entity_type_bundle_info
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->walletStorage = $entity_type_manager->getStorage('mcapi_wallet');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
    );
  }

  public function validateArgument($argument) {
    $current_user = \Drupal::currentUser();
    if ($current_user->hasPermission('view all transactions')) {
      return TRUE;
    }
    $wallet = $this->walletStorage->load($argument);
    if ($wallet) {
      $holder = $wallet->holder->entity;
      // Wallets are owned either by users or entities which implement EntityOwnerInterface
      $owner_id = $holder instanceOf UserInterface ? $holder->id() : $holder->getOwnerId();
      if ($current_user->id() == $owner_id) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
