<?php

namespace Drupal\mcapi;

use Drupal\mcapi\Entity\Transaction;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\ParamConverter\EntityConverter;
use Symfony\Component\Routing\Route;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Provides upcasting for the Transaction entity
 *
 * Example:
 *
 * path: '/transaction/{serial}'
 * options:
 *   parameters:
 *     mcapi_transaction:
 *       serial: TRUE
 *
 * If {serial} is 0, the transaction will be pulled from the tempstore
 */
class TransactionSerialConverter extends EntityConverter {

  /**
   * The tempstore service.
   *
   * @var Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempstore;

  /**
   * Constructs a new EntityConverter.
   *
   * @param PrivateTempStoreFactory $tempstore
   */
  public function __construct(PrivateTempStoreFactory $tempstore) {
    $this->tempStore = $tempstore;
  }

  /**
   * {@inheritdoc}
   */
  public function convert($value, $definition, $name, array $defaults) {
    if ($value) {
      $transaction_storage = \Drupal::entityTypeManager()
        ->getStorage('mcapi_transaction');
      $serials = $transaction_storage->getQuery()->accessCheck(TRUE)
        ->accessCheck(FALSE)
        ->condition('serial', $value)
        ->execute();
      if ($serial = reset($serials)) {
        return Transaction::load($serial);
      }
      throw new NotFoundHttpException();
    }

    // A $value of zero means that this is the are-you-sure page before the
    // transaction has been saved. The transaction is retrieved therefore not in
    // the normal way from the database but from the tempstore.
    return $this->tempStore
      ->get('TransactionForm')
      ->get('mcapi_transaction');
  }

  /**
   * {@inheritdoc}
   */
  public function applies($definition, $name, Route $route) {
    return $name == 'mcapi_transaction' && !empty($definition['serial']);
  }

}
