<?php

namespace Drupal\mcapi;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleUninstallValidatorInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use \Drupal\mcapi\Entity\Type;
use Drupal\mcapi\Entity\State;

/**
 * Validates module uninstall readiness based on transaction types and states
 */
class UninstallValidator implements ModuleUninstallValidatorInterface {
  use StringTranslationTrait;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new ContentUninstallValidator.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, TranslationInterface $string_translation) {
    $this->entityTypeManager = $entity_type_manager;
    $this->stringTranslation = $string_translation;
  }

  /**
   * {@inheritdoc}
   */
  public function validate($module) {
    $reasons = [];
    if ($module <> 'mcapi')return;
    foreach (State::loadMultiple() as $id => $mcapi_state) {
      $depends = $mcapi_state->getDependencies();
      if (isset($depends['module']) and in_array('mcapi', $depends['module'])) {
        $ids = $this->entityTypeManager->getStorage('mcapi_transaction')
          ->getQuery()
          ->accessCheck(FALSE)
          ->condition('state', $id)->execute();
        if ($ids) {
          $reasons[] = $this->t('@count transactions exist with state %state', ['@count' => count($ids), '%state' => $mcapi_state->label()]);
        }
      }
    }
    foreach (Type::loadMultiple() as $id => $mcapi_type) {
      $depends = $mcapi_type->getDependencies();
      if (isset($depends['module']) and in_array($module, $depends['module'])) {
        $ids = $this->entityTypeManager->getStorage('mcapi_transaction')
          ->getQuery()
          ->accessCheck(FALSE)
          ->condition('type', $id)->execute();
        if ($ids) {
          $reasons[] = $this->t('@count transactions exist with type %type', ['@count' => count($ids), '%type' => $mcapi_type->label()]);
        }
      }
    }
    if ($reasons) {
      $reasons[] = 'These transactions must be deleted manually.';
    }
    return $reasons;
  }

}
